import {
	forbiddenDisplayNamePatterns,
	guildNameRestrictions,
	reservedGuildNames,
} from "#environment/constants";

import type { validationResult } from "#types/lib";

function isValidGuildName(rawGuildName: string): validationResult {
	if (typeof rawGuildName !== "string") {
		return { valid: false, error: "Guild name must be a string" };
	}

	const guildName = rawGuildName.trim().normalize("NFC");

	if (!guildName) {
		return { valid: false, error: "Guild name is required" };
	}

	if (guildName.length < guildNameRestrictions.length.min) {
		return { valid: false, error: "Guild name is too short" };
	}

	if (guildName.length > guildNameRestrictions.length.max) {
		return { valid: false, error: "Guild name is too long" };
	}

	if (!guildNameRestrictions.regex.test(guildName)) {
		return { valid: false, error: "Guild name contains invalid characters" };
	}

	if (/^[._\s-]|[._\s-]$/.test(guildName)) {
		return {
			valid: false,
			error: "Guild name can't start or end with special characters or spaces",
		};
	}

	if (/\s{2,}/.test(guildName)) {
		return {
			valid: false,
			error: "Guild name cannot contain multiple consecutive spaces",
		};
	}

	if (reservedGuildNames.includes(guildName.toLowerCase())) {
		return {
			valid: false,
			error: "Guild name is reserved and cannot be used",
		};
	}

	for (const pattern of forbiddenDisplayNamePatterns) {
		if (pattern.test(guildName)) {
			return {
				valid: false,
				error: "Guild name contains invalid characters or patterns",
			};
		}
	}

	return { valid: true, value: guildName };
}

export { isValidGuildName };
